use crate::botv2::{
    domain::meme::change_auto_meme::change_auto_meme,
    init::{Context, Error},
};
use poise::CreateReply;
use tracing::instrument;

/// Enable/Disable auto answer available meme keywords
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(slash_command, prefix_command, category = "meme")]
pub async fn enable(
    ctx: Context<'_>,
    #[description = "Enable or diable "] enable: Option<bool>,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let embed: Result<
        Vec<poise::serenity_prelude::CreateEmbed>,
        crate::botv2::domain::meme::change_auto_meme::MemeChangeAutoMemeError,
    > = change_auto_meme(enable, ctx.author().id.get(), guild.get(), entity_name).await;
    let mut reply = CreateReply::default();
    if let Ok(embed) = embed {
        reply.embeds = embed;
    }
    if let Err(why) = ctx.send(reply).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
