use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::db::concour::Concour;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum UpdateConcourError {
    DoesntExist,
    FindError(String),
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn update_concour(
    server_id: u64,
    channel_id: u64,
    title: Option<String>,
    description: Option<String>,
    role_recopense: Option<u64>,
    banner: Option<String>,
) -> Result<Option<Concour>, UpdateConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(UpdateConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_none() {
        info!("Concour doesn't exist");
        return Err(UpdateConcourError::DoesntExist);
    }

    let mut concour = concour.unwrap();
    if let Some(title) = title {
        concour.title = title;
    }
    if let Some(description) = description {
        concour.description = description;
    }
    if let Some(role_recopense) = role_recopense {
        concour.role_recompense = role_recopense;
    }
    if let Some(banner) = banner {
        concour.banner = Some(banner);
    }

    match concour.update().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error updating concour");
            return Err(UpdateConcourError::UnknownError(
                "Error updating concour".to_string(),
            ));
        }
    }
    Ok(Some(concour))
}
