use poise::serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter};
use tracing::instrument;

use crate::db::user_image::User;

pub enum MemeChangeAutoMemeError {
    UserNotFound,
    UserCreateError,
    UserUpdateError,
    UserFindError,
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn change_auto_meme(
    enable: Option<bool>,
    user_id: u64,
    guild_id: u64,
    bot_entity: String,
) -> Result<Vec<CreateEmbed>, MemeChangeAutoMemeError> {
    let footer = CreateEmbedFooter::new(bot_entity.clone());
    let mut embed = CreateEmbed::new()
        .title("Enable/Disable auto meme answer")
        .footer(footer);
    let mut user_in_db: User = match User::find_by_server_id_user_id(&guild_id, &user_id).await {
        Ok(Some(user_in_db)) => user_in_db.clone(),
        Ok(None) => {
            let user_in_db = User::new(guild_id, user_id, false).unwrap();
            match user_in_db.create().await {
                Ok(_) => user_in_db,
                Err(e) => {
                    let embed = embed
                        .field("Could't create user in db", e.to_string(), false)
                        .color(colour::Color::RED);
                    return Ok(vec![embed]);
                }
            }
        }
        Err(e) => {
            let embed = embed
                .field("Error finding user image", e.to_string(), false)
                .color(colour::Color::RED);
            return Ok(vec![embed]);
        }
    };

    if let Some(new_status) = enable {
        if user_in_db.enable == new_status {
            let embed = embed.field(
                format!("Auto answer the same : {}", new_status),
                String::new(),
                false,
            );
            return Ok(vec![embed]);
        }
        user_in_db.enable = new_status;
        match user_in_db.update().await {
            Ok(_) => {
                embed = embed.field("Auto answer", new_status.to_string(), false);
            }
            Err(e) => {
                embed = embed.field("Couldn't update user in db", e.to_string(), false);
            }
        }
    } else {
        embed = embed.field("Auto answer", user_in_db.enable.to_string(), false);
    }
    return Ok(vec![embed]);
}
