use serde::{Deserialize, Serialize};
use tracing::instrument;

use crate::db::server_config::ServerConfig;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum GetServerConfigError {
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn get_server_config(
    server_id: u64,
) -> Result<Option<ServerConfig>, GetServerConfigError> {
    let server_config =
        match crate::db::server_config::ServerConfig::find_by_server_id(&server_id).await {
            Ok(server_config) => server_config,
            Err(err) => {
                tracing::error!(error = err.to_string(), "Error finding server config");
                return Err(GetServerConfigError::UnknownError(
                    "Error finding server config".to_string(),
                ));
            }
        };
    Ok(server_config)
}
